uniform sampler2D diffuseMap;
uniform sampler2D normalMap;

uniform vec3 lightIntensity;
uniform float specularPower;

varying vec3 localLightDirection;
varying vec3 localViewerDirection;

vec3 VecToColor( in vec3 vector ) {
	return (vector + 1.0) * 0.5;
}

vec3 ColorToVec( in vec3 color ) {
	return (color - 0.5) * 2.0;
}

/*
Generic light model function
*/
struct Light {
	vec3 direction;
	vec3 intensity;
	float specularPower;
};

struct Surface {
	vec3 normal;
	vec4 diffuseFactor;
	vec3 specularFactor;
};
	
vec4 getColor( in Surface surface, in Light light, in vec3 viewerDirection ) {
	float lightDistance = length( light.direction );
	vec3 normLightDirection = light.direction / lightDistance;
	vec3 normViewerDirection = normalize( viewerDirection );
	
	float attenuation = 1.0 / lightDistance;
	vec3 diffuseColor = surface.diffuseFactor * max( 0.0, dot( normLightDirection, surface.normal ) );
	
	vec3 specularColor = surface.specularFactor * pow( max( 0.0, dot( reflect( ^-normLightDirection, surface.normal ), normViewerDirection ) ), light.specularPower );
	vec3 finalColor = attenuation * light.intensity * (diffuseColor + specularColor);
	
	return vec4( finalColor, surface.diffuseFactor.a );
}

void main()
{
	vec4 diffuse = texture2D( diffuseMap, vec2( gl_TexCoord[ 0 ] ) );
	vec3 normal = ColorToVec( vec3( texture2D( normalMap, vec2( gl_TexCoord[ 0 ] ) ) ) );
	//Light light = Light( lightDirection, lightIntensity, 32.0 );
	Light light = Light( localLightDirection, lightIntensity, 32.0 );
	Surface surface = Surface( normal, diffuse, vec3( 1.0, 1.0, 1.0 ) );
	
	gl_FragColor = getColor( surface, light, localViewerDirection );	
}
